# (c) Zygmunt Krynicki 2005, 2006, 2007
# Licensed under GPL, see COPYING for the whole text

from string import Template

def get_alternatives(postinst):
    """
    Attempts to extract alternatives registerd by a package based on the contents of the postinst script.
    NOTE: This code is a big HACK since we don't really understand or execute
    postinst shell scripts here. Some alternatives are not retrieved.
    """
    if not "update-alternatives" in postinst:
        return
    def strip_comment(line):
        i = line.find('#')
        if i == -1:
            return line
        else:
            return line[:i]

    shell_vars = dict()
    values_of_i = None
    name_of_i = None
    def expand_values(*text_list):
        #logging.debug("About to perform substitution on %s using %s and %s (for i-list)" % (text_list, shell_vars, values_of_i))
        if values_of_i is None:
            yield tuple([Template(text).safe_substitute(**shell_vars) for text in text_list])
        else:
            for i in values_of_i:
                shell_vars[name_of_i] = i
                yield tuple([Template(text).safe_substitute(**shell_vars) for text in text_list])
    def one_element_tuple_to_value(it):
        for item in it:
            if len(item) == 1:
                yield item[0]
            else:
                yield item
    def nice_expand(*text_list):
        return set(one_element_tuple_to_value(expand_values(*text_list)))
    for line in postinst.replace("\\\n","").splitlines():
        line = strip_comment(line).strip()
        if line == "":
            continue
        words = line.split()
        # this ugly hack is capable of detecting both
        # variable=value as well as variable="list of values"
        if '=' in words[0] and "==" not in words[0]:
            name = words[0][:words[0].index('=')]
            value = line[line.index('=')+1:].strip('\'"')
            shell_vars[name] = value
            #logging.debug("Based on line: %s" % line)
            #logging.debug("Variable binding established: %s=%s" % (name, value))
        # this ugly hack is capable of detecting
        # for something in a list of values; do ...
        elif len(words) > 3 and words[0] == 'for' and words[2] == 'in' and ';' in line:
            name_of_i = words[1]
            values_of_i = " ".join(nice_expand (line[len('for %s in' % name_of_i):line.index(';')])).split()
            #logging.debug("Based on line: %s" % line)
            #logging.debug("Iterable variable binding established %s=%s" % (name_of_i, values_of_i))
        # this ugly hack is capable of detecting
        # for something in a list of values\n
        elif len(words) > 3 and words[0] == 'for' and words[2] == 'in' and ';' not in line:
            name_of_i = words[1]
            values_of_i = " ".join(nice_expand(" ".join(words[3:]))).split()
            #logging.debug("Based on line: %s" % line)
            #logging.debug("Iterable variable binding established %s=%s" % (name_of_i, values_of_i))
        else:
            if words[0] == "update-alternatives":
                words = words[1:]
                while len(words) > 0:
                    if words[0] == "--install":
                        name = words[1]
                        target = words[3]
                        words = words[5:]
                    elif words[0] == "--slave":
                        name = words[1]
                        target = words[3]
                        words = words[4:]
                    # all remaining options are ignored
                    elif words[0] in ("--set", "--remove"):
                        words = words[3:]
                        continue
                    elif words[0] in ("--verbose", "--quiet", "--test", "--help", "--all"):
                        words = words[1:]
                        continue
                    elif words[0] in ("--altdir", "--admindir", "--list", "--display", "--auto", "--config", "--remove-all"):
                        words = words[2:]
                        continue
                    # sometimes || true or redirection follows the last argument
                    elif words[0] in (";", "||") or words[0].startswith(">"):
                        break
                    else:
                        #logging.error("%s: aborting analisys of update-alternatives on argument '%s', line: %s" % (
                        #    package.filename, words[0], line))
                        break
                    # sometimes pathnames are quoted
                    name = name.strip('\'"')
                    target = target.strip('\'"')
                    for new_name, new_target in nice_expand(name, target):
                        if not new_name.startswith('/'):
                            new_name = "/" + new_name
                        if not new_target.startswith('/'):
                            new_target = "/" + new_target
                        if '$' in new_name or '$' in new_target:
                            pass
                            #logging.error("%s: FIXME alternatives need manual analisys: %s -> %s" % (
                            #    package.filename, new_name, new_target))
                        else:
                            yield new_name, new_target
