# (c) Zygmunt Krynicki 2005, 2006, 2007
# Licensed under GPL, see COPYING for the whole text

import warnings
warnings.filterwarnings("ignore", "apt API not stable yet", FutureWarning)
import apt_inst, apt_pkg, os.path
from file_info import *
from generic_package import Package

from ArchiveCrawler import dumbExtractControl
print dumbExtractControl

def load(pathname):
    """Load a debian package from the given pathname and return it as a generic package"""
    package = Package()
    def get_control_file(name):
        """ Returns the contents of given file in debian/ or None if it does not exits """
        return dumbExtractControl(pathname, name)
    sections = apt_pkg.ParseSection(get_control_file('control'))
    package.file_name = os.path.basename(pathname)
    package.arch = sections["Architecture"]
    package.name = sections["Package"]
    package.postinst = get_control_file('postinst')
    try:
        parts = pathname.split(os.path.sep)
        package.repo = parts[parts.index("pool")+1]
    except ValueError:
        package.repo = None
    package.contents = []
    def extract_cb(kind, name, target, mode, uid, gid, size, mtime, major, minor):
        if kind == "FILE":
            package.contents.append(FileInfo(name, mode, uid, gid, size, mtime))
        elif kind == "DIR":
            package.contents.append(DirectoryInfo(name, mode, uid, gid, size, mtime))
        elif kind == "SYMLINK":
            package.contents.append(SymbolicLinkInfo(name, target, mode, uid, gid, size, mtime))
        elif kind == "HARDLINK":
            package.contents.append(HardLinkInfo(name, target, mode, uid, gid, size, mtime))
        elif kind == "FIFO":
            package.contents.append(FifoInfo(name, mode, uid, gid, size, mtime))
        else:
            raise Exception("unsupported file kind: %s" % kind)
    try:
        apt_inst.debExtract(open(pathname), extract_cb, "data.tar.gz")
    except:
        try:
            apt_inst.debExtract(open(pathname), extract_cb, "data.tar.bz2")
        except:
            apt_inst.debExtract(open(pathname), extract_cb, "data.tar.lzma")
    return package
