#!/usr/bin/python

import apt_pkg
import os
import os.path
import sys
sys.path.insert(0, "../")

import unittest
import logging
import shutil
import subprocess

from CommandDataExtractor import CommandDataExtractor

class testCommandDataExtractor(unittest.TestCase):

    debfiles_done = "./data/command.p"

    def rm(self, f):
        if os.path.exists(f):
            os.unlink(f)
    def setUp(self):
        self.rm(self.debfiles_done)
        self.rm("./data/scan.data")

    def testExtractorSimple(self):
        extractor = CommandDataExtractor("./aptroot",
                                         "./pool",
                                         "./data")
        extractor.extract()
        self.assert_(os.path.exists("./data/scan.data"))
        self.assert_(len(open("./data/scan.data").readlines()) > 3)

    def testExtractorOrphan(self):
        extractor = CommandDataExtractor("./aptroot",
                                         "./pool",
                                         "./data")
        extractor.extract()
        self.assert_(len(open("./data/scan.data").readlines()) > 2)
        self.assert_("gnome-session-remove" in open(extractor.cmd_data_f).read())
        # now simulate archive removal of a pkg
        b = "./pool/main/g/gnome-session/gnome-session_2.14.1-0ubuntu11_i386.deb"
        os.rename(b, b+".xxx")
        # clean orphaned desktop files and make sure that the file really
        # got removed
        extractor.extract()
        os.rename(b+".xxx", b)        
        self.assert_(len(open("./data/scan.data").readlines()) > 2)
        self.assert_("gnome-session-remove" not in open(extractor.cmd_data_f).read())

    def testExtractorCleanPkg(self):
        extractor = CommandDataExtractor("./aptroot",
                                         "./pool",
                                         "./data")
        # simulate superseeding
        open("./data/scan.data","w").write("i386|main|gnome-session|gnome-session-remove,gnome-session-save,gnome-session-properties,x-session-manager\n")
        extractor.extract()
        self.assert_(open("./data/scan.data").read().count("i386|main|gnome-session|") == 1)
        self.assert_("gnome-wm" in open(extractor.cmd_data_f).read())

if __name__ == "__main__":
    subprocess.call(["(cd pool; ./get_pkgs.sh)"],shell=True)
    logging.basicConfig(level=logging.DEBUG)
    apt_pkg.init()
    #unittest.main(defaultTest="testCommandDataExtractor.testExtractorOrphan")
    unittest.main()
