#!/usr/bin/python

import apt_pkg
import os
import sys
sys.path = [os.path.normpath(os.path.join(os.getcwd(),"../"))] + sys.path

import glob
import unittest
import logging
from ArchiveCrawler import ArchiveCrawler

class testArchiveCrawler(unittest.TestCase):

    pkgs_found_file = "pkgs-found"
    debfiles_done = "./data/debfiles-done.p"

    def rm(self, f):
        if os.path.exists(f):
            os.unlink(f)
        
    def setUp(self):
        # update the cache only once
        crawler = ArchiveCrawler("./aptroot", "./pool", "./actions", "./data", "i386")
        crawler.updateCache()
        # remove pkgs-found file to make tests meaningful
        self.rm(self.pkgs_found_file)
        self.rm(self.debfiles_done)

    def testCrawlerSimple(self):
        def callback_helper(crawler, debfile, pkg, ver, arch, component):
            self._i += 1
            return True
        crawler = ArchiveCrawler("./aptroot", "./pool", "./actions", "./data", "i386")
        # test if we allow only callables
        try:
            crawler.registerCallback(self._i)
        except Exception, e:
            got_exception = True
        # now register a real callback
        self._i = 0
        crawler.registerCallback(callback_helper)
        # run
        crawler.crawl()
        self.assert_(os.path.isfile(self.pkgs_found_file))
        self.assert_(self._i > 0)
        self.assert_(got_exception)
        self.rm(self.debfiles_done)
        del self._i

    def testCrawlerSkipDone(self):
        crawler = ArchiveCrawler("./aptroot", "./pool", "./actions", "./data", "i386")
        crawler.crawl()
        self.assert_(len(crawler.debfiles_done) > 0)
        # delete the found-pkgs marker file, crawl again
        self.rm(self.pkgs_found_file)
        crawler.crawl()
        self.assert_(not os.path.exists(self.pkgs_found_file))
        self.rm(self.debfiles_done)
        
    def testCrawlerSkipPersistant(self):
        crawler = ArchiveCrawler("./aptroot", "./pool", "./actions", "./data", "i386")
        crawler.crawl()
        # delete the found-pkgs marker file, create new crawler, crawl again,
        # see if we skip the done files
        self.rm(self.pkgs_found_file)
        crawler = ArchiveCrawler("./aptroot", "./pool", "./actions", "./data", "i386")
        crawler.crawl()
        self.assert_(not os.path.exists(self.pkgs_found_file))

    def testFindOrphanedFiles(self):
        crawler = ArchiveCrawler("./aptroot", "./pool", "./actions", "./data", "i386")
        crawler.crawl()
        crawler.debfiles_done.add("./pool/main/h/hello/hello_2.1.1-3_i386.deb")
        print crawler.findOrphanedFiles()
        self.assert_(len(crawler.findOrphanedFiles()) == 1)

    def testFindOrphanedPackages(self):
        crawler = ArchiveCrawler("./aptroot", "./pool", "./actions", "./data", "i386")
        crawler.crawl()
        print crawler.findOrphanedPackages()
        self.assert_(len(crawler.findOrphanedPackages()) == 0)
        crawler.debfiles_done.add("./pool/main/h/hello/hello_2.1.1-3_i386.deb")
        print crawler.findOrphanedPackages()
        self.assert_(len(crawler.findOrphanedPackages()) == 1)

    def testFindObsoletedPackages(self):
        # FIXME: this test is pretty useless
        crawler = ArchiveCrawler("./aptroot", "./pool", "./actions", "./data", "i386")
        crawler.crawl()
        #print "obsolete: ", crawler.findObsoletedPackages()
        self.assert_(len(crawler.findObsoletedPackages()) > 0)

    def testArches(self):
        for arch in ('amd64', 'i386'):
            crawler = ArchiveCrawler("./aptroot", "./pool", "./actions", "./data", arch)
            crawler.updateCache()
            self.assert_(len(glob.glob(os.path.join(crawler.aptroot,"var/lib/apt/lists")+"/*%s*" % arch)) > 0)

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    apt_pkg.init()
    unittest.main()
