#!/usr/bin/python

import apt_pkg
import os
import os.path
import sys
sys.path = [os.path.normpath(os.path.join(os.getcwd(),"../"))] + sys.path

import unittest
import logging
import shutil
import subprocess

from DesktopDataExtractor import DesktopDataExtractor

class TestDesktopDataExtractor(unittest.TestCase):

    debfiles_done = "./data/desktop.p"

    def rm(self, f):
        if os.path.exists(f):
            os.unlink(f)
    def setUp(self):
        self.rm(self.debfiles_done)
        try:
            shutil.rmtree("./data/menu-data")
        except:
            pass

    def testExtractorSimple(self):
        extractor = DesktopDataExtractor("./aptroot",
                                         "./pool",
                                         "./data")
        extractor.extract()
        # see if extraction works
        self.assertTrue(os.path.exists(os.path.join(extractor.menu_data,"synaptic.desktop")))
        # see if lzma works
        self.assertTrue(os.path.exists(os.path.join(extractor.menu_data,"ooo-math.desktop")))
        # gnome-about is blacklisted
        self.assertTrue(not os.path.exists(os.path.join(extractor.menu_data,"gnome-about.desktop")))
        # check if icon extraction works
        self.assertTrue(os.path.exists(os.path.join(extractor.menu_data,"icons", "_usr_share_pixmaps_python2.5.xpm")))

    def testExtractorSimpleMaverick(self):
        extractor = DesktopDataExtractor("./aptroot-maverick",
                                         "./pool",
                                         "./data")
        extractor.extract()
        # check if we have baobab
        self.assertTrue(os.path.exists(os.path.join(extractor.menu_data,"baobab.desktop")))


    def testExtractorOrphan(self):
        extractor = DesktopDataExtractor("./aptroot",
                                         "./pool",
                                         "./data")
        extractor.extract()
        self.assert_(os.path.exists(os.path.join(extractor.menu_data,"session-properties.desktop")))

        # now simulate archive removal of a pkg
        b = "./pool/main/g/gnome-session/gnome-session_2.22.1.1-0ubuntu2_i386.deb"
        os.rename(b, b+".xxx")
        # clean orphaned desktop files and make sure that the file really
        # got removed
        extractor.extract()
        self.assert_(not os.path.exists(os.path.join(extractor.menu_data,"session-properties.desktop")))
        os.rename(b+".xxx", b)

if __name__ == "__main__":
    if len(sys.argv) > 1 and sys.argv[1] == '-v':
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)        
    subprocess.call(["(cd pool; ./get_pkgs.sh)"],shell=True)
    apt_pkg.init()
    unittest.main()
